# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

########################################
# DESCRIPTION: auxiliary functions for CR simulation analysis
# WORKING DIR: ../
########################################

generate_lcmpop <- function(props, lam, N){
  J <- NCOL(lam)
  K <- length(props)
  tab <- as.data.frame(lapply(1:J, FUN = function(j)rep(factor(NA, levels = 1:2, labels = c('No', 'Yes')), N)))
  names(tab) <- paste('L', 1:J, sep = '')
  z <- sample(1:K, size = N, prob = props, replace = T)
  for (j in 1:J){
    tab[, j] <- factor(c('No', 'Yes')) [ rbinom(n = length(z), size = 1, prob = lam[z, j]) + 1 ]
  }
  tab[apply(tab, MARGIN = 1, FUN = function(x) any(x == 'Yes')), ]
}

probs_zero_by_class <- function(pi, lambda){
  apply(1-lambda, MARGIN = 1, FUN = prod)
}

probs_zero <- function(pi, lambda){
  a <- probs_zero_by_class(pi, lambda)
  sum(a * pi)
}

