# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

quick <- Sys.getenv("MK_QUICK_RUN") == "TRUE"
#debugging <- TRUE
# quick <- TRUE
library(snowfall)
sfInit(parallel = TRUE, cpus = 20)
sfLibrary(LCMCR)
sfSource('src/fn_CR.R')

##############################
# Short run debugging parameters. Not used in batch mode.
if (exists("debugging")){
  trial_par <- list( ntrials = 10, samples = 10, Ns = c(2e3L, 2e4L, 2e5L), th = 10, burnin = 10, filename = 'output/test.rds')
  lcm_par <- list( pi = c(0.9, 0.1),
    lambda = matrix(c(
      0.033, 0.099, 0.132, 0.033,
      0.825, 0.759, 0.99, 0.693
    ),  nrow = 4, byrow = TRUE)
  )
} else {
  x <- jsonlite::read_json(commandArgs(trailingOnly = TRUE), simplifyVector = TRUE)
  lcm_par <- x$lcm_par
  if (quick){
    trial_par <- list( ntrials = 10, samples = 10, Ns = c(2e3L, 2e4L, 2e5L), th = 10, burnin = 10, filename = x$trial_par$filename)
  } else {
    trial_par <- x$trial_par
  }
}
##############################
sfExport('trial_par', 'lcm_par')
res <- list(summaries = list(), meta = list())
for(N in trial_par$Ns) {
  batch <- sfLapply(1:trial_par$ntrials, fun = function(j, N) {
      set.seed(j + N)
      a <- generate_lcmpop(props = lcm_par$pi, lam = lcm_par$lambda, N = N)
      x <- lcmCR(captures = a, in_list_label='Yes', not_in_list_label='No',
                 buffer_size = trial_par$samples,
                 thinning = trial_par$th, seed = j, K = 10, verbose = F)
      x$Set_Trace('prob_zero')
      lcmCR_PostSampl(x, burnin = trial_par$burnin, thinning = trial_par$th,
                      clear_buffer = TRUE, output = FALSE)
      p0 <- x$Get_Trace('prob_zero')
      list(mean_n = mean(NROW(a)), mean_p0 = mean(p0), quant = quantile(p0, prob = c(0.5, 0.025, 0.975, 0.25, 0.75)))
  }, N = N)
  for (j in 1:trial_par$ntrials) {
    res$summaries[[ paste('N', N, sep = '_') ]] [[ paste('trial', j, sep = '_')]] <- batch[[j]]
  }
  res$meta = list(lcm_par = lcm_par, trial_par = trial_par, date = date())
  saveRDS(res, trial_par$filename)
}

