# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

########################################
# DESCRIPTION: Run single-runs for each batches/repeated_xx.json file.
# WORKING DIR: 
# INPUT (FILES):
#   - batches/repeated_[3-4][a|b|c].json
# OUTPUT (FILES):
#   - output/single_runs.rds
########################################

# quick <- TRUE
quick <- Sys.getenv("MK_QUICK_RUN") == "TRUE"
library(snowfall)
sfInit(parallel = TRUE, cpus = 20)
sfLibrary(LCMCR)
sfSource('src/fn_CR.R')

a <- expand.grid(
  c("batches/repeated_3a.json",
    "batches/repeated_3b.json",
    "batches/repeated_4a.json",
    "batches/repeated_4b.json",
    "batches/repeated_4c.json"),
  c(2000,1e4,1e5)
)
sfExport('quick')

res <- sfApply(a, margin = 1, fun = function(X){
  set.seed(123)
  d <- jsonlite::read_json(X[1], simplifyVector = TRUE)
  if (quick){
    mc_par <- list(nsam = 10, th = 1, bi = 1)
  } else {
    mc_par <- list(nsam = d$trial_par$samples,
                   th = d$trial_par$th, bi = d$trial_par$burnin)
  }
  N <- X[2]
  a <- generate_lcmpop(props = d$lcm_par$pi, lam = d$lcm_par$lambda, N = N)
  x <- lcmCR(captures = a, in_list_label='Yes', not_in_list_label='No',
             buffer_size = mc_par$nsam,
             thinning = mc_par$th, seed = 123, K = 10, verbose = F)
  x$Set_Trace('prob_zero')
  lcmCR_PostSampl(x, burnin = mc_par$bi, samples = mc_par$nsam,
                  thinning = mc_par$th,
                  clear_buffer = TRUE, output = FALSE)
  p0 <- x$Get_Trace('prob_zero')
  name <- unlist(strsplit(X[1],'[\\._]'))[2]
  list(sam_p0 = p0, n = NROW(a), exp = name, N = N, lcm_par = d$lcm_par)
})
names(res) <- sapply(res, function(x)paste0(x$exp, '_', x$N))

saveRDS(res, file = 'output/single_runs.rds')
