# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

#quick <- TRUE
quick <- Sys.getenv("MK_QUICK_RUN") == "TRUE"

library(LCMCR)
library(snowfall)
library(coda)
sfInit(parallel = TRUE, cpus = 20)
sfLibrary(LCMCR)
fn <- new.env()
source('src/fn_CR.R', local = fn)
sfSource('src/fn_CR.R')

# example 1
examples <- list(
  example_1 = list(
    lambda = structure(c(0.2475, 0.7425, 0.2475, 0.7425), .Dim = c(2L, 2L)),
    pi = c(0.5, 0.5),
    p0S = 0.219
  ),
  example_2 = list(
    lambda =  structure(c(0.099, 0.759, 0.132, 0.99), .Dim = c(2L, 2L)),
    pi = c(0.9, 0.1)
  )
)

Ns <- c(2e3, 1e4, 1e5)
if (quick){
  burnin <- 1
  samples <- 20
  th <- 1
} else {
  burnin <- 50000
  samples <- 2000
  th <- 100
}

res <- sapply(examples, function(par) {
  set.seed(123)
  sfExport('burnin', 'samples', 'th', 'par')
  simul <- sfSapply(Ns,
    function(N){
      a <- generate_lcmpop(par$pi, par$lambda, N)
      x <- lcmCR(captures = a, in_list_label='Yes', not_in_list_label='No',buffer_size = samples,
                 thinning = th, seed = 2, K = 2, verbose = F)
      x$Set_Trace(c('prob_zero','nuK'))
      lcmCR_PostSampl(x, burnin = burnin, thinning = th, samples = samples,
                      clear_buffer = TRUE, output = FALSE)
      n <- NROW(a)
      nhat_ind <- exp(glm(Freq ~., family = poisson, data = as.data.frame(xtabs(~., a))[-1,])$coefficient[1])
      p0hat_ind <- 1 - n/(nhat_ind + n)
      list(samps = x$Get_Trace('prob_zero'),  pi_samps = x$Get_Trace('nuK'),
           p0hat_ind = p0hat_ind, n = n, N = N, iter = x$Get_Iteration())
    },
    USE.NAMES = TRUE, simplify = FALSE
  )
  names(simul) <- paste('N',format(Ns), sep = '_')
  list(
              simul = simul,
              par = par,
              p0T = fn$probs_zero(par$pi, par$lambda),
              niter = samples * th,
              thinning = th,
              burnin = burnin,
              date = date())
  },
  USE.NAMES = TRUE, simplify = FALSE
)


saveRDS(res, file = 'output/two_list_examples_single_run.rds')


